<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Applicator;

use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteTransitionEnum;
use SM\Factory\FactoryInterface;
use Webmozart\Assert\Assert;

class QuoteStateMachineTransitionApplicator implements QuoteStateMachineTransitionApplicatorInterface
{
    public function __construct(
        private readonly FactoryInterface $stateMachineFactory,
    ) {
    }

    public function accept(OrderInterface $data): OrderInterface
    {
        $stateMachine = $this->stateMachineFactory->get($data, QuoteTransitionEnum::GRAPH);

        Assert::true(
            $stateMachine->can(QuoteTransitionEnum::TRANSITIONS_ACCEPT),
            sprintf('Order with %s token cannot be accepted.', $data->getTokenValue()),
        );

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_ACCEPT);

        return $data;
    }

    public function acceptWithResupply(OrderInterface $data): OrderInterface
    {
        $stateMachine = $this->stateMachineFactory->get($data, QuoteTransitionEnum::GRAPH);

        Assert::true(
            $stateMachine->can(QuoteTransitionEnum::TRANSITIONS_ACCEPT_RESUPPLY),
            sprintf('Order with %s token cannot be accepted with resupply.', $data->getTokenValue()),
        );

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_ACCEPT_RESUPPLY);

        return $data;
    }

    public function deny(OrderInterface $data): OrderInterface
    {
        $stateMachine = $this->stateMachineFactory->get($data, QuoteTransitionEnum::GRAPH);

        Assert::true(
            $stateMachine->can(QuoteTransitionEnum::TRANSITIONS_DENY),
            sprintf('Order with %s token cannot be denied.', $data->getTokenValue()),
        );

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_DENY);

        return $data;
    }
}
