<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Controller;

use Dedi\SyliusQuotePlugin\Api\Command\RemoveQuote;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Messenger\MessageBusInterface;
use Symfony\Component\Translation\Exception\NotFoundResourceException;

class DeleteQuoteAction extends AbstractController
{
    public function __construct(
        private readonly MessageBusInterface $commandBus,
        private readonly QuoteRepositoryInterface $quoteRepository,
    ) {
    }

    public function __invoke(Request $request): Response
    {
        $tokenValue = $request->attributes->get('tokenValue');
        if (null === $tokenValue) {
            throw new BadRequestException('Token value is missing');
        }

        $quote = $this->quoteRepository->findOneByTokenValue($tokenValue);
        if ($quote === null) {
            throw new NotFoundResourceException(sprintf('Quote with token value "%s" not found', $tokenValue));
        }

        $this->commandBus->dispatch(new RemoveQuote($tokenValue));

        return new JsonResponse(status: Response::HTTP_NO_CONTENT);
    }
}
