<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Controller;

use Dedi\SyliusQuotePlugin\Api\Command\RemoveInternalCommentFromQuote;
use Dedi\SyliusQuotePlugin\Repository\InternalCommentRepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Messenger\MessageBusInterface;
use Symfony\Component\Translation\Exception\NotFoundResourceException;

class DeleteQuoteInternalCommentAction extends AbstractController
{
    public function __construct(
        private readonly MessageBusInterface $commandBus,
        private readonly InternalCommentRepositoryInterface $internalCommentRepository,
    ) {
    }

    public function __invoke(Request $request): Response
    {
        $commentId = $request->attributes->get('commentId');
        $tokenValue = $request->attributes->get('tokenValue');
        if (null === $commentId || null === $tokenValue) {
            throw new BadRequestException('Comment ID or token value is missing');
        }

        $comment = $this->internalCommentRepository->findByIdAndQuoteTokenValue($commentId, $tokenValue);
        if ($comment === null) {
            throw new NotFoundResourceException('Internal comment not found');
        }

        $this->commandBus->dispatch(new RemoveInternalCommentFromQuote($tokenValue, $commentId));

        return new JsonResponse(status: Response::HTTP_NO_CONTENT);
    }
}
