<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Controller;

use Dedi\SyliusQuotePlugin\Api\Command\RemoveItemFromQuote;
use Sylius\Bundle\ApiBundle\Exception\OrderItemNotFoundException;
use Sylius\Component\Core\Repository\OrderItemRepositoryInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Messenger\MessageBusInterface;

class DeleteQuoteItemAction extends AbstractController
{
    public function __construct(
        private readonly MessageBusInterface $commandBus,
        private readonly OrderItemRepositoryInterface $orderItemRepository,
    ) {
    }

    public function __invoke(Request $request): Response
    {
        $itemId = $request->attributes->get('itemId');
        $tokenValue = $request->attributes->get('tokenValue');
        if (null === $itemId || null === $tokenValue) {
            throw new OrderItemNotFoundException();
        }

        $orderItem = $this->orderItemRepository->findOneByIdAndCartTokenValue($itemId, $tokenValue);
        if ($orderItem === null) {
            throw new OrderItemNotFoundException();
        }

        $this->commandBus->dispatch(new RemoveItemFromQuote($tokenValue, $itemId));

        return new JsonResponse(status: Response::HTTP_NO_CONTENT);
    }
}
