<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\AddInternalCommentToQuote;
use Dedi\SyliusQuotePlugin\Entity\InternalCommentInterface;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Sylius\Component\Core\Model\AdminUserInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class AddInternalCommentToQuoteHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly RepositoryInterface $adminUserRepository,
        private readonly FactoryInterface $internalCommentFactory,
    ) {
    }

    public function __invoke(AddInternalCommentToQuote $addInternalCommentToQuote): void
    {
        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($addInternalCommentToQuote->orderTokenValue);

        Assert::notNull($quote);

        $quoteInformation = $quote->getQuoteInformation();

        Assert::notNull($quoteInformation);

        $adminUser = $this->adminUserRepository->find($addInternalCommentToQuote->user);

        $comment = $addInternalCommentToQuote->comment;

        if (null === $comment) {
            return;
        }

        /** @var InternalCommentInterface $internalComment */
        $internalComment = $this->internalCommentFactory->createNew();
        $internalComment->setComment($comment);

        if ($adminUser instanceof AdminUserInterface) {
            $internalComment->setUser($adminUser);
        }

        $internalComment->setQuoteInformation($quote->getQuoteInformation());

        $quoteInformation->addInternalComment($internalComment);
    }
}
