<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\AddItemToQuote;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Entity\OrderItemInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInformationInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Sylius\Component\Core\Factory\CartItemFactoryInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Sylius\Component\Core\Repository\ProductVariantRepositoryInterface;
use Sylius\Component\Order\Modifier\OrderItemQuantityModifierInterface;
use Sylius\Component\Order\Modifier\OrderModifierInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class AddItemToQuoteHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly ProductVariantRepositoryInterface $productVariantRepository,
        private readonly OrderModifierInterface $orderModifier,
        private readonly CartItemFactoryInterface $cartItemFactory,
        private readonly OrderItemQuantityModifierInterface $orderItemQuantityModifier,
        private readonly FactoryInterface $quoteItemInformationFactory,
    ) {
    }

    public function __invoke(AddItemToQuote $addItemToQuote): OrderInterface
    {
        /** @var ProductVariantInterface|null $productVariant */
        $productVariant = $this->productVariantRepository->findOneBy(['code' => $addItemToQuote->productVariantCode]);

        Assert::notNull($productVariant);

        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($addItemToQuote->orderTokenValue);

        Assert::notNull($quote);

        /** @var OrderItemInterface $quoteItem */
        $quoteItem = $this->cartItemFactory->createNew();
        $quoteItem->setVariant($productVariant);
        $quoteItem->setProductName($productVariant->getProduct()?->getName());
        $quoteItem->setVariantName($productVariant->getName());

        $this->orderItemQuantityModifier->modify($quoteItem, $addItemToQuote->quantity);
        $this->orderModifier->addToOrder($quote, $quoteItem);

        /** @var QuoteItemInformationInterface $quoteItemInformation */
        $quoteItemInformation = $this->quoteItemInformationFactory->createNew();

        $quoteItemInformation->setUnitPrice($addItemToQuote->unitPrice);
        $quoteItemInformation->setOrderItem($quoteItem);
        $quoteItem->setQuoteItemInformation($quoteItemInformation);

        return $quote;
    }
}
