<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\CompleteQuote;
use Dedi\SyliusQuotePlugin\Enum\QuoteTransitionEnum;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use SM\Factory\FactoryInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class CompleteQuoteHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly FactoryInterface $stateMachineFactory,
    ) {
    }

    public function __invoke(CompleteQuote $completeQuote): OrderInterface
    {
        $orderTokenValue = $completeQuote->orderTokenValue;

        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($orderTokenValue);

        Assert::notNull($quote, sprintf('Quote with %s token has not been found.', $orderTokenValue));
        Assert::notNull($quote->getCustomer(), 'Please enter your email before completing the order.');

        $stateMachine = $this->stateMachineFactory->get($quote, QuoteTransitionEnum::GRAPH);

        Assert::true(
            $stateMachine->can(QuoteTransitionEnum::TRANSITIONS_CREATE),
            sprintf('Order with %s token cannot be completed.', $orderTokenValue),
        );

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_CREATE);

        return $quote;
    }
}
