<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\PlaceQuote;
use Dedi\SyliusQuotePlugin\Converter\QuoteToCartConverterInterface;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteInformationStateEnum;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Dedi\SyliusQuotePlugin\Specification\Order\IsQuoteSpecificationInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Sylius\Component\Core\Storage\CartStorageInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class PlaceQuoteHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly CartContextInterface $cartContext,
        private readonly ChannelContextInterface $channelContext,
        private readonly IsQuoteSpecificationInterface $isQuoteSpecification,
        private readonly OrderRepositoryInterface $orderRepository,
        private readonly QuoteToCartConverterInterface $quoteToCartConverter,
        private readonly CartStorageInterface $cartStorage,
    ) {
    }

    public function __invoke(PlaceQuote $placeQuote): OrderInterface
    {
        $orderTokenValue = $placeQuote->getOrderTokenValue();

        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($orderTokenValue);

        Assert::notNull($quote, sprintf('Quote with %s token has not been found.', $orderTokenValue));
        Assert::eq($quote->getQuoteState(), QuoteInformationStateEnum::STATE_ACCEPTED, sprintf('Order with %s token cannot be placed.', $orderTokenValue));

        /** @var OrderInterface $cart */
        $cart = $this->cartContext->getCart();
        if (!$cart->isEmpty() && !$this->isQuoteSpecification->isSatisfiedBy($cart)) {
            $this->orderRepository->remove($cart);
            $cart = $this->cartContext->getCart();
        }

        $this->quoteToCartConverter->convert($quote, $cart);
        $this->orderRepository->add($cart);

        /** @var ChannelInterface $channel */
        $channel = $this->channelContext->getChannel();
        $this->cartStorage->setForChannel($channel, $quote);

        return $quote;
    }
}
