<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\RemoveItemFromQuote;
use Dedi\SyliusQuotePlugin\Entity\OrderItemInterface;
use Doctrine\ORM\EntityManagerInterface;
use Sylius\Component\Order\Repository\OrderItemRepositoryInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class RemoveItemFromQuoteHandler
{
    public function __construct(
        private readonly OrderItemRepositoryInterface $orderItemRepository,
        private readonly EntityManagerInterface $entityManager,
    ) {
    }

    public function __invoke(RemoveItemFromQuote $removeItemFromQuote): void
    {
        /** @var OrderItemInterface|null $orderItem */
        $orderItem = $this->orderItemRepository->findOneByIdAndCartTokenValue(
            $removeItemFromQuote->itemId,
            $removeItemFromQuote->orderTokenValue,
        );

        Assert::notNull($orderItem);

        $quoteInformation = $orderItem->getQuoteItemInformation();

        $this->entityManager->remove($quoteInformation);
    }
}
