<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use Dedi\SyliusQuotePlugin\Api\Command\UpdateItemFromQuote;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Entity\OrderItemInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Sylius\Component\Core\Repository\OrderItemRepositoryInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class UpdateItemFromQuoteHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
        private readonly OrderItemRepositoryInterface $orderItemRepository,
    ) {
    }

    public function __invoke(UpdateItemFromQuote $updateItemFromQuote): OrderInterface
    {
        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($updateItemFromQuote->getOrderTokenValue());

        Assert::notNull($quote);

        /** @var OrderItemInterface|null $orderItem */
        $orderItem = $this->orderItemRepository->findOneByIdAndCartTokenValue(
            $updateItemFromQuote->getSubresourceId(),
            $quote->getTokenValue(),
        );

        Assert::notNull($orderItem);

        $quoteItemInformation = $orderItem->getQuoteItemInformation();

        if (null !== $updateItemFromQuote->quantity) {
            $quoteItemInformation->getOrderItem()->setQuantity($updateItemFromQuote->quantity);
        }

        if (null !== $updateItemFromQuote->unitPrice) {
            $quoteItemInformation->setUnitPrice($updateItemFromQuote->unitPrice);
        }

        return $quote;
    }
}
