<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Api\Handler;

use DateTime;
use Dedi\SyliusQuotePlugin\Api\Command\UpdateQuoteInformation;
use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepositoryInterface;
use Symfony\Component\Messenger\Attribute\AsMessageHandler;
use Webmozart\Assert\Assert;

#[AsMessageHandler]
class UpdateQuoteInformationHandler
{
    public function __construct(
        private readonly QuoteRepositoryInterface $quoteRepository,
    ) {
    }

    public function __invoke(UpdateQuoteInformation $updateQuoteInformation): OrderInterface
    {
        $tokenValue = $updateQuoteInformation->getOrderTokenValue();

        /** @var OrderInterface|null $quote */
        $quote = $this->quoteRepository->findCartByTokenValue($tokenValue);
        Assert::notNull($quote, sprintf('Quote with %s token has not been found.', $tokenValue));

        $quoteInformation = $quote->getQuoteInformation();
        Assert::notNull($quoteInformation, sprintf('Quote information not found for quote with %s token.', $tokenValue));

        if (null !== $updateQuoteInformation->getCustomerComment()) {
            $quoteInformation->setCustomerComment($updateQuoteInformation->getCustomerComment());
        }
        if (null !== $updateQuoteInformation->getInformationComment()) {
            $quoteInformation->setInformationComment(
                $updateQuoteInformation->getInformationComment(),
            );
        }
        if (null !== $updateQuoteInformation->getExpirationDate()) {
            $quoteInformation->setExpirationDate(new DateTime($updateQuoteInformation->getExpirationDate()));
        }
        if (null !== $updateQuoteInformation->getTotal()) {
            $quoteInformation->setTotal($updateQuoteInformation->getTotal());
        }

        return $quote;
    }
}
