<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Enum\OrderStateEnum;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Channel\Context\ChannelNotFoundException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Sylius\Component\Customer\Context\CustomerContextInterface;
use Sylius\Component\Order\Context\CartContextInterface;
use Sylius\Component\Order\Context\CartNotFoundException;
use Webmozart\Assert\Assert;

class CustomerAndChannelBasedCartContext implements CartContextInterface
{
    public function __construct(
        private readonly CustomerContextInterface $customerContext,
        private readonly ChannelContextInterface $channelContext,
        private readonly OrderRepositoryInterface $orderRepository,
    ) {
    }

    public function getCart(): OrderInterface
    {
        try {
            $channel = $this->channelContext->getChannel();
        } catch (ChannelNotFoundException) {
            throw new CartNotFoundException('Sylius was not able to find the cart, as there is no current channel.');
        }

        $customer = $this->customerContext->getCustomer();
        if (null === $customer) {
            throw new CartNotFoundException('Sylius was not able to find the cart, as there is no logged in user.');
        }
        Assert::isInstanceOf($channel, ChannelInterface::class);
        Assert::isInstanceOf($customer, CustomerInterface::class);

        /** @var OrderInterface|null $cart */
        $cart = $this->orderRepository->findLatestNotEmptyCartByChannelAndCustomer($channel, $customer);

        if (null === $cart || $cart->getQuoteState() !== OrderStateEnum::STATE_NONE) {
            throw new CartNotFoundException('Sylius was not able to find the cart for currently logged in user.');
        }

        return $cart;
    }
}
